#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include "error.h"
#include "cdbslib.h"
#include "config.h"
#include "system.h"
#include "errormsg.h"
#include "cdbserror.h"

//=======================================================================
// LOGGING LIBRARY routine
//   cdbs_error, cdbs_warn, cdbs_info, cdbs_debug, cdbs_message
//=======================================================================

int        cdbs_errorflag = NO;
void    (*cdbs_errhandler)(int, const char *) = NULL;

/* CDBS_ERROR -- Print cdbs error message and exit
 *
 * i: code	error code
 * i: fmt	format for arguments
 */

void cdbs_error (const int code, const char *fmt, ...)
{
    va_list	ptr;
    char	message[SZ_LINE+1], arglist[SZ_LINE+1];

    /* Check for out of bounds error codes */

    if (code < 0 || code > MAXERR)
	cdbs_error (UNKNOWN, "%d", code);

    /* Format message arguments as a string */

    va_start (ptr, fmt);
    vsprintf (arglist, fmt, ptr);
    va_end (ptr);

    /* Write message to output */

    safecopy (message, error_msg[code], SZ_LINE);
    safecat (message, " (", SZ_LINE);
    safecat (message, arglist, SZ_LINE);
    safecat (message, ")\n", SZ_LINE);

    cdbs_message (ERROR_MSG, message);
}

/* CDBS_WARN -- Print cdbs warning message to output file
 *
 * i: code	error code
 * i: fmt	format for arguments
 */

void cdbs_warn (const int code, const char *fmt, ...)

{
    va_list	ptr;
    char	message[SZ_LINE+1], arglist[SZ_LINE+1];

    /* Check for out of bounds error codes */

    if (code < 0 || code > MAXERR)
	cdbs_error (UNKNOWN, "%d", code);

    /* Format message arguments as a string */

    va_start (ptr, fmt);
    vsprintf (arglist, fmt, ptr);
    va_end (ptr);

    /* Write message to output */

    safecopy (message, error_msg[code], SZ_LINE);
    safecat (message, " (", SZ_LINE);
    safecat (message, arglist, SZ_LINE);
    safecat (message, ")\n", SZ_LINE);

    cdbs_message (WARN_MSG, message);
}

/* CDBS_INFO -- Print cdbs informational message
 *
 * i: code	error code
 * i: fmt	format for arguments
 */

void cdbs_info (const int code, const char *fmt, ...)
{
    va_list	ptr;
    char	message[SZ_LINE+1], arglist[SZ_LINE+1];

    /* Check for out of bounds error codes */

    if (code < 0 || code > MAXERR)
	cdbs_error (UNKNOWN, "%d", code);

    /* Format message arguments as a string */

    va_start (ptr, fmt);
    vsprintf (arglist, fmt, ptr);
    va_end (ptr);

    /* Write message to output */

    safecopy (message, error_msg[code], SZ_LINE);
    safecat (message, " (", SZ_LINE);
    safecat (message, arglist, SZ_LINE);
    safecat (message, ")\n", SZ_LINE);

    cdbs_message (INFO_MSG, message);
}

/******************************************************************
 * cdbs_debug -- Print cdbs debug message, if enabled
 *
 * The environment variable CDBS_DEBUG must be defined in order
 * to display debug-level messages.
 *
 * i: fmt	format for arguments
 *
 * 09/13/02  MSwam  first version
 *
 ******************************************************************/
void cdbs_debug (const char *fmt, ...)
{
    va_list	ptr;
    char	message[SZ_LINE+1], arglist[SZ_LINE+1], *envvar;
    static int cdbs_debugmode = NO;
    static int cdbs_envtest = YES;

    /* check environment for debug switch once */
    if (cdbs_envtest == YES) {
        cdbs_envtest = NO;
        envvar = getenv ("CDBS_DEBUG");
        if (envvar != NULL) {
            cdbs_debugmode = YES;
        }
    }

    if (cdbs_debugmode == NO) return;

    /* Format message arguments as a string */

    va_start (ptr, fmt);
    vsprintf (arglist, fmt, ptr);
    va_end (ptr);

    /* Write message to output */

    safecopy (message, "-DEBUG-", SZ_LINE);
    safecat (message, " (", SZ_LINE);
    safecat (message, arglist, SZ_LINE);
    safecat (message, ")\n", SZ_LINE);

    cdbs_message (3, message);
}

/* CDBS_MESSAGE -- Print cdbs message to stdout
 *
 * i: code	error code
 * i: fmt	format for arguments
 */

void cdbs_message (const int flag, const char *message)
{
    if (cdbs_errhandler == NULL) {
	fputs (message, ERRORLOG);
    } else {
	(*cdbs_errhandler) (flag, message);
    }

    if (flag == WARN_MSG)
        cdbs_errorflag = YES;

    if (flag == ERROR_MSG)
        exit (ERROR_EXIT);

}

